from setuptools import setup
from setuptools import find_packages

setup(
    name='csnl-vae-olaralex',
    version='1.37dev',
    author="Alex Olar",
    author_email="olaralex@caesar.elte.hu",
    packages=find_packages(),
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description='Variational Autoencoders with different architectures and test files',
)